/***************************************************************************
    File:                : Sample7.cpp 
	Description			 :
                           -------------------
    begin                : 
    copyright            : (C) 2002 by Eduardo Sampaio
    email                : ejtsr@cin.ufpe.br
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "Global\Global.h"
#include "GameManager.h"
#include "GameState.h"
#include "GraphicEngine\VideoEngine.h"
#include "GraphicEngine\Surface.h"
#include "GraphicEngine\Sprite.h"
#include "GraphicEngine\Tileset.h"
#include "InputEngine\CommonStructs.h"
#include "InputEngine\Keyboard.h"

#include "SoundEngine\SoundEngine.h"
#include "SoundEngine\SoundEnvironment.h"


CF16VSurface*			 g_pbackground	   = NULL;
CF16VStereoEnvironment*  g_pEnvironment	   = NULL;
CF16VSound*				 g_pSound		   = NULL;
CF16VSound*				 g_pSound2		   = NULL;

class CF16VSample7GameState : public CF16VGameState{

protected:


public:
	CF16VSample7GameState();
	virtual ~CF16VSample7GameState();
	virtual void	IncludeGameState();
	virtual void	RemoveGameState();
	virtual void KeyDown( int iKey );
	virtual HRESULT ProcessFrame(void);



};


CF16VSample7GameState::CF16VSample7GameState() {
	ReceiveKeyboardEvent(TRUE);

}

CF16VSample7GameState::~CF16VSample7GameState() {
}

void	CF16VSample7GameState::IncludeGameState() {

	// Init sound engine
	CF16VSoundEngine* soundEngine = GetGameManager()->InitSoundEngine();

	// Create a new stereo environment 
	if ( soundEngine) g_pEnvironment = soundEngine->CreateStereoEnvironment();

	// Create sounds
	if (g_pEnvironment) g_pSound = new CF16VSound(soundEngine);
	if (g_pEnvironment) g_pSound2 = new CF16VSound(soundEngine);

	// Init sound1
	g_pSound->CreateSegmentFromFile("water.wav",false);
	// Sets the number of repeats
	g_pSound->SetRepeats(F16V_REPEAT_INFINITE);
	// Init sound2
	g_pSound2->CreateSegmentFromFile("bird.wav",false);

	// Create and load background surface
	g_pbackground = new CF16VSurface();
	g_pbackground->Create(GetVideoEngine(),"bkgrnd.jpg");

	// Hides the mouse cursor
	GetVideoEngine()->SetCursorVisible(FALSE);
}

void	CF16VSample7GameState::RemoveGameState() {
	SAFE_DELETE(g_pbackground);
	SAFE_DELETE(g_pSound);
	SAFE_DELETE(g_pSound2);
}

void	CF16VSample7GameState::KeyDown( int iKey ) {
	if( iKey == F16VK_ESCAPE) {
		GetGameManager()->Exit(0);

	}
	if (iKey == F16VK_F) {
		if(g_pSound && g_pEnvironment) { 
			if (!g_pSound->IsPlaying()) g_pSound->Play(g_pEnvironment); //if not playing the sound, play it
			else g_pSound->Stop(); // otherwise stop it
		}
	}

	if (iKey == F16VK_G) {
		if(g_pSound2 && g_pEnvironment) {
			if (!g_pSound2->IsPlaying()) g_pSound2->Play(g_pEnvironment);
			else g_pSound2->Stop();
		}
	}

}


HRESULT CF16VSample7GameState::ProcessFrame(void) {
	CF16VVideoEngine *ve = GetGameManager()->GetVideoEngine();
	CF16VSurface* back = ve->GetBackBuffer();

	g_pbackground->Draw(back, 0,0, NULL,false); // Draws the background



	return 0;
}




int WINAPI WinMain(HINSTANCE hinstance,
				   HINSTANCE hprevinstance,
				   LPSTR	lpcmdline,
				   int ncmdshow) {



	CF16VGameManager *gm = CF16VGameManager::Instanciate();

	CF16VSample7GameState state;
	gm->AddGameState(&state);
//	gm->GetVideoEngine()->CreateFullScreen(800,600,16);
	gm->GetVideoEngine()->CreateWindowed(800,600);


	return 0;
	
}
