/***************************************************************************
    File:                : Sample10.cpp 
	Description			 :
                           -------------------
    begin                : 
    copyright            : (C) 2002 by Eduardo Sampaio
    email                : ejtsr@cin.ufpe.br
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "Global\Global.h"
#include "GameManager.h"
#include "GameState.h"
#include "GraphicEngine\VideoEngine.h"
#include "GraphicEngine\Surface.h"
#include "GraphicEngine\Tileset.h"
#include "InputEngine\CommonStructs.h"
#include "InputEngine\Keyboard.h"
#include "InputEngine\Mouse.h"
#include "GraphicEngine\Isometric\Map.h"
#include "GraphicEngine\Isometric\MapCell.h"
#include "GraphicEngine\Isometric\MouseMap.h"
#include "GraphicEngine\Isometric\Scroller.h"
#include "GraphicEngine\Isometric\TilePlotter.h"


//tamanho do mapa
#define MAPWIDTH 32
#define MAPHEIGHT 64


class Sample10MapCell;

CF16VMapCell**    g_mapCells; //array de celulas
CF16VSurface*  g_mouseMapSurf;
CF16VTileset*  g_cursor;
CF16VTileset*  g_tiles;
CF16VMap		g_map;
CHAR			g_buffer[32];





class Sample10MapCell : public CF16VMapCell {

public:
	virtual RECT*	GetExtentRect();
	virtual HRESULT	 PaintCell(CF16VSurface *dest, LONG x, LONG y, UINT layer);
	Sample10MapCell();

protected:

};

class CF16VSample10GameState : public CF16VGameState{

protected:
	

public:
	CF16VSample10GameState();
	virtual ~CF16VSample10GameState();
	virtual void	IncludeGameState();
	virtual void	RemoveGameState();
	virtual void	KeyDown( int iKey );
	virtual void MouseButtonUp( int iButton );
	virtual HRESULT ProcessFrame(void);

};


Sample10MapCell::Sample10MapCell() {

}

RECT*	Sample10MapCell::GetExtentRect() {
	if(g_tiles) return g_tiles->GetTileInfo(0)->GetDstExtentRect(); // pega o retangulo do tileset
	else return NULL;
}


HRESULT	 Sample10MapCell::PaintCell(CF16VSurface *dest, LONG x, LONG y, UINT layer) {
	if(dest) return g_tiles->DrawFast(dest, x, y, 3); // desenha o tile de umero 3
	else return -1;
}



CF16VSample10GameState::CF16VSample10GameState() {
	ReceiveKeyboardEvent(TRUE);
	ReceiveMouseEvent(TRUE);

}

CF16VSample10GameState::~CF16VSample10GameState() {
}

void	CF16VSample10GameState::IncludeGameState() {

	// Cria o array de celulas
	g_mapCells = new CF16VMapCell*[MAPWIDTH*MAPHEIGHT];
	for (int i = 0; i < MAPWIDTH*MAPHEIGHT; i++ ) {
		g_mapCells[i] = new Sample10MapCell;

	}
	
	// Muda o modo do mouse para ser exclusivo desta aplicao
	CF16VGameManager::GetGameManager()->GetInputEngine()->GetMouse()->ChangeMouseMode(TRUE,TRUE); // exclusive , foreground

	g_tiles = new CF16VTileset();
	g_tiles->Create(CF16VGameManager::GetGameManager()->GetVideoEngine(),"Tiles.bmp");
	g_tiles->SetTileInfoFromSurface();

	g_mouseMapSurf = new CF16VSurface();
	g_mouseMapSurf->Create(CF16VGameManager::GetGameManager()->GetVideoEngine(), "MouseMap.bmp");

	g_cursor = new CF16VTileset();
	g_cursor->Create(CF16VGameManager::GetGameManager()->GetVideoEngine(), "Cursor.bmp");
	g_cursor->SetTileInfoFromSurface();

	// Cria um novo mapa passando o tipo do mapa, tamanha e array de celulas e o nmero de layers
	g_map.Create(CF16VGameManager::GetGameManager()->GetVideoEngine(), ISOMAP_SLIDE, MAPWIDTH,MAPHEIGHT, g_mapCells, 1);
	g_map.GetMouseMap()->Create(g_mouseMapSurf);

	//  acrescenta uma "borda" no  mapa
	RECT tempRect = { -100, -100, 100, 100 }; 
	g_map.GetScroller()->CalcWorldSpace(&tempRect);
	g_map.GetScroller()->CalcAnchorSpace();
	g_map.GetScroller()->GetAnchor()->x = -100;
	g_map.GetScroller()->GetAnchor()->y = -100;
	
    
}

void	CF16VSample10GameState::RemoveGameState() {
	SAFE_DELETE(g_tiles);
	SAFE_DELETE(g_mouseMapSurf);
	SAFE_DELETE(g_cursor);
	for (int i =0 ; i < MAPWIDTH*MAPHEIGHT; i++) SAFE_DELETE( *(g_mapCells+i)); 
	SAFE_DELETE_ARRAY(g_mapCells);
}

void	CF16VSample10GameState::KeyDown( int iKey ) {
	if( iKey == F16VK_ESCAPE) {
		CF16VGameManager::GetGameManager()->Exit(0);

	}
	if (iKey == F16VK_RETURN) {
		if( CF16VGameManager::GetGameManager()->GetInputEngine()->GetKeyBoard()->GetKeyDown(F16VK_LMENU)) {
				CF16VGameManager::GetGameManager()->GetVideoEngine()->ChangeVideoMode(!CF16VGameManager::GetGameManager()->GetVideoEngine()->IsFullScreen());
		}

	}
	if( iKey ==  F16VK_F1) { // muda o tipo do mapa
		g_map.SetMapType(ISOMAP_SLIDE);
		g_map.GetScroller()->SetHWrapMode(WRAPMODE_CLIP);
		g_map.GetScroller()->SetVWrapMode(WRAPMODE_CLIP);
	
	}
	if (iKey ==  F16VK_F2) { // muda o tipo do mapa
		g_map.SetMapType(ISOMAP_STAGGERED);
		g_map.GetScroller()->SetHWrapMode(WRAPMODE_CLIP);
		g_map.GetScroller()->SetVWrapMode(WRAPMODE_CLIP);

		
		RECT tempRect = { -100, -100, 100, 100 }; 
		g_map.GetScroller()->CalcWorldSpace(&tempRect);
		g_map.GetScroller()->CalcAnchorSpace();
		g_map.GetScroller()->GetAnchor()->x = -100;
		g_map.GetScroller()->GetAnchor()->y = -100;


	}
	if (iKey ==  F16VK_F3) {
		g_map.SetMapType(ISOMAP_STAGGERED);
		g_map.GetScroller()->SetHWrapMode(WRAPMODE_WRAP);
		g_map.GetScroller()->SetVWrapMode(WRAPMODE_CLIP);
		g_map.GetScroller()->GetAnchorSpace()->top+= g_map.GetMouseMap()->GetHeight()/2;
		g_map.GetScroller()->GetAnchorSpace()->bottom-= g_map.GetMouseMap()->GetHeight()/2;
		g_map.GetScroller()->GetAnchorSpace()->left+= g_map.GetMouseMap()->GetWidth()/2;
		g_map.GetScroller()->GetAnchorSpace()->right-= g_map.GetMouseMap()->GetWidth()/2;

	}
	
	if( iKey  == F16VK_F4) {
		g_map.SetMapType(ISOMAP_DIAMOND);
		g_map.GetScroller()->SetHWrapMode(WRAPMODE_CLIP);
		g_map.GetScroller()->SetVWrapMode(WRAPMODE_CLIP);

		RECT tempRect = { -100, -100, 100, 100 }; 
		g_map.GetScroller()->CalcWorldSpace(&tempRect);
		g_map.GetScroller()->CalcAnchorSpace();
		g_map.GetScroller()->GetAnchor()->x = -100;
		g_map.GetScroller()->GetAnchor()->y = -100;


	}

}

void CF16VSample10GameState::MouseButtonUp( int iButton ) {
			//grab mouse position
			POINT ptMouse;
			CF16VGameManager::GetGameManager()->GetInputEngine()->GetMouse()->GetMousePos(&ptMouse.x, &ptMouse.y);

}


HRESULT CF16VSample10GameState::ProcessFrame(void) {
	CF16VVideoEngine *ve = CF16VGameManager::GetGameManager()->GetVideoEngine();
	CF16VSurface* back = ve->GetBackBuffer();

	POINT ptMouse, ptCursor;
	CF16VGameManager::GetGameManager()->GetInputEngine()->GetMouse()->GetMousePos(&ptMouse.x, &ptMouse.y);

	LONG width = back->GetWidth();
	LONG height = back->GetHeight();
	LONG scrollx =0;
	LONG scrolly = 0;
	//left scroll?
	if(ptMouse.x<10) scrollx = ptMouse.x-10;

	//upward scroll?
	if(ptMouse.y<8) scrolly=ptMouse.y-8;

	//right scroll?
	if(ptMouse.x>=width-10) scrollx=ptMouse.x-(width-10);

	//downward scroll?
	if(ptMouse.y>=height-8) scrolly=ptMouse.y-(height-8);
	
	back->Fill(0);

	g_map.ScrollFrame(scrollx, scrolly); // realiza o scroll no mapa
	g_map.UpdateFrame(); // Repinta todo o mapa

	POINT ptFine;
	ptCursor = g_map.GetMouseMap()->MapMouse(ptMouse, &ptFine);

	//clip the cursor to valid map coordinates
	if(ptCursor.x<0) ptCursor.x=0;
	if(ptCursor.y<0) ptCursor.y=0;
	if(ptCursor.x>(MAPWIDTH-1)) ptCursor.x=MAPWIDTH-1;
	if(ptCursor.y>(MAPHEIGHT-1)) ptCursor.y=MAPHEIGHT-1;
	
	LONG pX = ptCursor.x;
	LONG pY = ptCursor.y;
	
	ptCursor = g_map.GetTilePlotter()->PlotTile(ptCursor);
	ptCursor = g_map.GetScroller()->WorldToScreen(ptCursor); // converte de tile para pixels na tela
	g_cursor->DrawFast(back, ptCursor.x,  ptCursor.y); // desenha o cursor "vermelho"

	// imprime informaes dos tiles
	memcpy(g_buffer,"\0",32);
	_snprintf(g_buffer,31, "TileX: %d  TileY: %d", pX, pY);

	ve->GetFont()->DrawText(back, 700,20, g_buffer, strlen(g_buffer));
	memcpy(g_buffer,"\0",32);
	_snprintf(g_buffer,31, "IntileX: %d  IntileY: %d", ptFine.x, ptFine.y);
	ve->GetFont()->DrawText(back, 700,30, g_buffer, strlen(g_buffer));

	return 0;
}


int WINAPI WinMain(HINSTANCE hinstance,
				   HINSTANCE hprevinstance,
				   LPSTR	lpcmdline,
				   int ncmdshow) {



	CF16VGameManager *gm = CF16VGameManager::Instanciate();

	CF16VSample10GameState state;
	gm->AddGameState(&state);
//	gm->GetVideoEngine()->CreateFullScreen(800,600,16);
	gm->GetVideoEngine()->CreateWindowed(800,600);


	return 0;
	
}
